#pragma once

#include "MidiEvent.h"
#include <memory>
#include <string>

class MetaMidiEvent : public MidiEvent
{
public:
    enum class Type
    {
        UNSET,
        TRACK_NAME,
        SET_TEMPO,
        TIME_SIGNATURE
    };

private:

    static const int META_SET_TEMPO = 0x51; // 0101 0001
    static const int META_TRACK_NAME = 0x3; // 0000 0011
    static const int META_TIME_SIG = 0x58; // 0101 1000
    Type mType;
    std::string mLabel;
    int mValue;

public:

    MetaMidiEvent();
    static std::shared_ptr<MetaMidiEvent> Create();

    void SetValue(int value);

    void SetIsTrackName();

    void SetIsSetTempo();

    void SetIsTimeSignature();

    void SetLabel(const std::string& label);

    static bool IsTrackName(char c);

    static bool IsSetTempo(char c);

    static bool IsTimeSignature(char c);
};

using MetaMidiEventPtr = std::shared_ptr<MetaMidiEvent>;
