#include "MidiChannelEvent.h"

MidiChannelEvent::MidiChannelEvent()
    : MidiEvent(),
      mType(Type::NONE),
      mValue0(0),
      mValue1(0)
{

}

std::shared_ptr<MidiChannelEvent> MidiChannelEvent::Create()
{
    return std::make_shared<MidiChannelEvent>();
}

void MidiChannelEvent::SetType(Type type)
{
    mType = type;
}
MidiChannelEvent::Type MidiChannelEvent::GetType()
{
    return mType;
}
void MidiChannelEvent::SetValues(int value0, int value1)
{
    mValue0 = value0;
    mValue1 = value1;
}
int MidiChannelEvent::GetValue0()
{
    return mValue0;
}
int MidiChannelEvent::GetValue1()
{
    return mValue1;
}

bool MidiChannelEvent::IsControllerEvent(int c)
{
    return (c & CONTROLLER) == CONTROLLER;
}

bool MidiChannelEvent::IsNoteOnEvent(int c)
{
    return (c & NOTE_ON) == NOTE_ON;
}

bool MidiChannelEvent::IsNoteOffEvent(int c)
{
    return (c & NOTE_OFF) == NOTE_OFF;
}

bool MidiChannelEvent::IsNoteAfterTouchEvent(int c)
{
    return (c & NOTE_AFTERTOUCH) == NOTE_AFTERTOUCH;
}

bool MidiChannelEvent::IsProgramEvent(int c)
{
    return (c & PROGRAM) == PROGRAM;
}

bool MidiChannelEvent::IsChannelAftertouchEvent(int c)
{
    return (c & CHANNEL_AFTERTOUCH) == CHANNEL_AFTERTOUCH;
}

bool MidiChannelEvent::IsPitchbendEvent(int c)
{
    return (c & PITCH_BEND) == PITCH_BEND;
}

bool MidiChannelEvent::IsStatusByte(int c)
{
    return IsControllerEvent(c) || IsNoteOnEvent(c) ||
            IsNoteOffEvent(c) || IsNoteAfterTouchEvent(c) ||
            IsProgramEvent(c) || IsChannelAftertouchEvent(c) ||
            IsPitchbendEvent(c);
}
