#pragma once

#include "MidiDocument.h"
#include "MetaMidiEvent.h"
#include "MidiChannelEvent.h"
#include <string>

class MidiReader
{
    static constexpr const char TrackChunkLabel[] = "MTrk";
    static constexpr const char HeaderLabel[] = "MThd";
    MidiDocument mDocument;
    unsigned mTrackByteCount;
    MidiChannelEvent::Type mLastChannelEventType;

public:

    MidiReader();

    void Read(const std::string& path);

private:

    bool ProcessHeader(std::ifstream& file);

    bool ProcessTimeDivision(std::ifstream& file);

    bool ProcessTrackChunk(std::ifstream& file, bool debug=false);

    bool ProcessEvent(std::ifstream& file, MidiTrack& track);

    bool ProcessMidiEventData(std::ifstream& file, MidiChannelEvent& event, char c);
    bool ProcessMidiEventData(std::ifstream& file, MidiChannelEvent& event);

    bool ProcessMidiChannelEvent(std::ifstream& file, char firstByte, MidiChannelEvent& event);
    bool ProcessMetaEvent(std::ifstream& file, MetaMidiEvent& event);

    int ProcessTimeDelta(std::ifstream& file);

    bool ProcessTrackNameMetaEvent(std::ifstream& file, MetaMidiEvent& event);
    bool ProcessSetTempoMetaEvent(std::ifstream& file, MetaMidiEvent& event);
    bool ProcessTimeSignatureMetaEvent(std::ifstream& file, MetaMidiEvent& event);
};
