#include "UnixSocketInterface.h"

#include <iostream>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include "HttpResponse.h"

UnixSocketInterface::UnixSocketInterface()
	: mOpeningHandles(),
	  mBufferSize(1024)
{

}

UnixSocketInterface::~UnixSocketInterface()
{

}

std::shared_ptr<UnixSocketInterface> UnixSocketInterface::Create()
{
	return std::make_shared<UnixSocketInterface>();
}

void UnixSocketInterface::CreateSocket(SocketPtr socketPtr)
{
	mOpeningHandles[socketPtr] = socket(AF_INET, SOCK_STREAM, 0);
}

void UnixSocketInterface::Listen(SocketPtr socket)
{
    if(mOpeningHandles[socket] < 0)
    {
		 std::cerr << "Error opening socket" << std::endl;
		 return;
    }

	int port = static_cast<int>(socket->GetPort());
    struct sockaddr_in serv_addr;
    memset(&serv_addr, 0, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    serv_addr.sin_addr.s_addr = INADDR_ANY;
    serv_addr.sin_port = htons(port);

    int result = bind(mOpeningHandles[socket], (struct sockaddr *)&serv_addr, sizeof(serv_addr));
    if(result< 0)
    {
		std::cerr << "Error binding socket" << std::endl;
		return;
    }

    listen(mOpeningHandles[socket], 5);
}

void UnixSocketInterface::Run(SocketPtr socket)
{
    if(mOpeningHandles[socket] < 0)
    {
		 std::cerr << "Error opening socket" << std::endl;
		 return;
    }

    struct sockaddr_in cli_addr;
    socklen_t clilen = sizeof(cli_addr);
    while(true)
    {
    	SocketHandle new_socket_handle = accept(mOpeningHandles[socket],
    			(struct sockaddr *) &cli_addr, &clilen);
        if (new_socket_handle < 0)
        {
			std::cerr << "Error on accept" << std::endl;
			return;
        }

        char buffer[mBufferSize] = {0};
        int n = read(new_socket_handle, buffer, mBufferSize);
        if (n < 0)
        {
			std::cerr << "Error on read" << std::endl;
			return;
        }
        socket->SetMessage(buffer);
        std::cout << "Here is the message: " << buffer << std::endl;

        HttpResponse response;
        response.SetBody("Hello world!");

        std::string response_message = response.ToString();
        n = write(new_socket_handle, response_message.c_str(), response_message.length());
        if (n < 0)
        {
			 std::cerr << "Error on write" << std::endl;
			 return;
        }
        close(new_socket_handle);
    }
}
