#pragma once

#include <vector>
#include <memory>

#include "Keyboard.h"
#include "Screen.h"
#include "KeyboardEvent.h"
#include "PaintEvent.h"
#include "MouseEvent.h"
#include "WindowManager.h"
#include "UiEvent.h"
#include "AbstractDesktopApp.h"

class DesktopManager
{
	std::vector<ScreenPtr> mScreens;
	WindowManagerPtr mWindowManager;
	KeyboardPtr mKeyboard;
	AbstractDesktopAppPtr mMainApplication;
	bool mModified;

public:

	DesktopManager();

	~DesktopManager();

	static std::shared_ptr<DesktopManager> Create();

	void SetWindowManager(WindowManagerPtr windowManager);

	void SetMainApp(AbstractDesktopAppPtr mainApp);

	AbstractDesktopAppPtr GetMainApp();

	WindowManagerPtr GetWindowManager();

	KeyboardPtr GetKeyboard();

	bool IsModified();

	void SetIsModified(bool modified);

	void SetKeyboard(KeyboardPtr keyboard);

	void OnUiEvent(UiEventPtr event);

	void OnKeyboardEvent(KeyboardEventPtr keyboardEvent);

	void OnMouseEvent(MouseEventPtr mouseEvent);

	void OnPaintEvent(PaintEventPtr paintEvent);


};

using DesktopManagerPtr = std::shared_ptr<DesktopManager>;
